@@ -1,3 +1,5 @@
+bool IsAnsiComment(const char *Data,int Size);
+
 bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
 {
   if (!MainComment)
@@ -9,22 +11,28 @@ bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
   if (OldFormat)
   {
     Seek(SFXSize+SIZEOF_OLDMHD,SEEK_SET);
-    CmtLength=GetByte()+(GetByte()<<8);
+    CmtLength=GetByte();
+    CmtLength+=(GetByte()<<8);
   }
   else
 #endif
   {
     if (NewMhd.Flags & MHD_COMMENT)
     {
+      // Old style (RAR 2.9) archive comment embedded into the main 
+      // archive header.
       Seek(SFXSize+SIZEOF_MARKHEAD+SIZEOF_NEWMHD,SEEK_SET);
       ReadHeader();
     }
     else
     {
+      // Current (RAR 3.0+) version of archive comment.
       Seek(SFXSize+SIZEOF_MARKHEAD+NewMhd.HeadSize,SEEK_SET);
       return(SearchSubBlock(SUBHEAD_TYPE_CMT)!=0 && ReadCommentData(CmtData,CmtDataW)!=0);
     }
 #ifndef SFX_MODULE
+    // Old style (RAR 2.9) comment header embedded into the main 
+    // archive header.
     if (CommHead.HeadCRC!=HeaderCRC)
     {
       Log(FileName,St(MLogCommHead));
@@ -49,7 +57,8 @@ bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
 #ifdef NOCRYPT
       return(false);
 #else
-      UnpCmtLength=GetByte()+(GetByte()<<8);
+      UnpCmtLength=GetByte();
+      UnpCmtLength+=(GetByte()<<8);
       CmtLength-=2;
       DataIO.SetCmt13Encryption();
 #endif
@@ -91,19 +100,20 @@ bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
     }
   }
 #endif
-#if defined(_WIN_32) && !defined(_WIN_CE)
+#if defined(_WIN_ALL) && !defined(_WIN_CE)
   if (CmtData->Size()>0)
   {
     size_t CmtSize=CmtData->Size();
-    OemToCharBuff((char *)CmtData->Addr(),(char *)CmtData->Addr(),(DWORD)CmtSize);
+    char *DataA=(char *)CmtData->Addr();
+    OemToCharBuffA(DataA,DataA,(DWORD)CmtSize);
 
     if (CmtDataW!=NULL)
     {
       CmtDataW->Alloc(CmtSize+1);
       CmtData->Push(0);
-      CharToWide((char *)CmtData->Addr(),CmtDataW->Addr(),CmtSize+1);
+      CharToWide(DataA,CmtDataW->Addr(),CmtSize+1);
       CmtData->Alloc(CmtSize);
-      CmtDataW->Alloc(strlenw(CmtDataW->Addr()));
+      CmtDataW->Alloc(wcslen(CmtDataW->Addr()));
     }
   }
 #endif
@@ -142,7 +152,7 @@ size_t Archive::ReadCommentData(Array<byte> *CmtData,Array<wchar> *CmtDataW)
       CmtDataW->Alloc(CmtSize+1);
       CharToWide((char *)CmtData->Addr(),CmtDataW->Addr(),CmtSize+1);
       CmtData->Alloc(CmtSize);
-      CmtDataW->Alloc(strlenw(CmtDataW->Addr()));
+      CmtDataW->Alloc(wcslen(CmtDataW->Addr()));
     }
   return(CmtSize);
 }
@@ -168,6 +178,9 @@ void Archive::ViewComment()
 
 
 #ifndef SFX_MODULE
+// Used for archives created by old RAR versions up to and including RAR 2.9.
+// New RAR versions store file comments in separate headers and such comments
+// are displayed in ListNewSubHeader function.
 void Archive::ViewFileComment()
 {
   if (!(NewLhd.Flags & LHD_COMMENT) || Cmd->DisableComment || OldFormat)
@@ -213,3 +226,5 @@ void Archive::ViewFileComment()
   }
 }
 #endif
+
+
