@@ -2,13 +2,16 @@
 
 MKDIR_CODE MakeDir(const char *Name,const wchar *NameW,bool SetAttr,uint Attr)
 {
-#ifdef _WIN_32
-  int Success;
-  if (WinNT() && NameW!=NULL && *NameW!=0)
-    Success=CreateDirectoryW(NameW,NULL);
-  else
-    Success=CreateDirectory(Name,NULL);
-  if (Success)
+#ifdef _WIN_ALL
+  BOOL RetCode;
+    if (WinNT() && NameW!=NULL && *NameW!=0)
+      RetCode=CreateDirectoryW(NameW,NULL);
+    else
+      if (Name!=NULL)
+        RetCode=CreateDirectoryA(Name,NULL);
+      else
+        return(MKDIR_BADPATH);
+  if (RetCode!=0) // Non-zero return code means success for CreateDirectory.
   {
     if (SetAttr)
       SetFileAttr(Name,NameW,Attr);
@@ -18,23 +21,30 @@ MKDIR_CODE MakeDir(const char *Name,const wchar *NameW,bool SetAttr,uint Attr)
   if (ErrCode==ERROR_FILE_NOT_FOUND || ErrCode==ERROR_PATH_NOT_FOUND)
     return(MKDIR_BADPATH);
   return(MKDIR_ERROR);
-#endif
-#ifdef _EMX
-#ifdef _DJGPP
-  if (mkdir(Name,(Attr & FA_RDONLY) ? 0:S_IWUSR)==0)
 #else
-  if (__mkdir(Name)==0)
+
+  // No Unicode in the rest of function, so Name must be not NULL.
+  if (Name==NULL)
+    return(MKDIR_BADPATH);
 #endif
-  {
-    if (SetAttr)
-      SetFileAttr(Name,NameW,Attr);
-    return(MKDIR_SUCCESS);
-  }
-  return(errno==ENOENT ? MKDIR_BADPATH:MKDIR_ERROR);
+
+#ifdef _EMX
+  #ifdef _DJGPP
+    if (mkdir(Name,(Attr & FA_RDONLY) ? 0:S_IWUSR)==0)
+  #else
+    if (__mkdir(Name)==0)
+  #endif
+    {
+      if (SetAttr)
+        SetFileAttr(Name,NameW,Attr);
+      return(MKDIR_SUCCESS);
+    }
+    return(errno==ENOENT ? MKDIR_BADPATH:MKDIR_ERROR);
 #endif
+
 #ifdef _UNIX
   mode_t uattr=SetAttr ? (mode_t)Attr:0777;
-  int ErrCode=Name==NULL ? -1:mkdir(Name,uattr);
+  int ErrCode=mkdir(Name,uattr);
   if (ErrCode==-1)
     return(errno==ENOENT ? MKDIR_BADPATH:MKDIR_ERROR);
   return(MKDIR_SUCCESS);
@@ -42,55 +52,31 @@ MKDIR_CODE MakeDir(const char *Name,const wchar *NameW,bool SetAttr,uint Attr)
 }
 
 
-bool CreatePath(const char *Path,const wchar *PathW,bool SkipLastName)
+bool CreatePath(const char *Path,bool SkipLastName)
 {
-#if defined(_WIN_32) || defined(_EMX)
+  if (Path==NULL || *Path==0)
+    return(false);
+
+#if defined(_WIN_ALL) || defined(_EMX)
   uint DirAttr=0;
 #else
   uint DirAttr=0777;
 #endif
-#ifdef UNICODE_SUPPORTED
-  bool Wide=PathW!=NULL && *PathW!=0 && UnicodeEnabled();
-#else
-  bool Wide=false;
-#endif
-  bool IgnoreAscii=false;
+  
   bool Success=true;
 
-  const char *s=Path;
-  for (int PosW=0;;PosW++)
+  for (const char *s=Path;*s!=0;s=charnext(s))
   {
-    if (s==NULL || s-Path>=NM || *s==0)
-      IgnoreAscii=true;
-    if (Wide && (PosW>=NM || PathW[PosW]==0) || !Wide && IgnoreAscii)
+    if (s-Path>=NM)
       break;
-    if (Wide && PathW[PosW]==CPATHDIVIDER || !Wide && *s==CPATHDIVIDER)
+
+    if (*s==CPATHDIVIDER)
     {
-      wchar *DirPtrW=NULL,DirNameW[NM];
-      if (Wide)
-      {
-        strncpyw(DirNameW,PathW,PosW);
-        DirNameW[PosW]=0;
-        DirPtrW=DirNameW;
-      }
       char DirName[NM];
-      if (IgnoreAscii)
-        WideToChar(DirPtrW,DirName);
-      else
-      {
-#ifndef DBCS_SUPPORTED
-        if (*s!=CPATHDIVIDER)
-          for (const char *n=s;*n!=0 && n-Path<NM;n++)
-            if (*n==CPATHDIVIDER)
-            {
-              s=n;
-              break;
-            }
-#endif
-        strncpy(DirName,Path,s-Path);
-        DirName[s-Path]=0;
-      }
-      if (MakeDir(DirName,DirPtrW,true,DirAttr)==MKDIR_SUCCESS)
+      strncpy(DirName,Path,s-Path);
+      DirName[s-Path]=0;
+
+      if (MakeDir(DirName,NULL,true,DirAttr)==MKDIR_SUCCESS)
       {
 #ifndef GUI
         mprintf(St(MCreatDir),DirName);
@@ -100,19 +86,78 @@ bool CreatePath(const char *Path,const wchar *PathW,bool SkipLastName)
       else
         Success=false;
     }
-    if (!IgnoreAscii)
-      s=charnext(s);
   }
-  if (!SkipLastName && !IsPathDiv(*PointToLastChar(Path)))
-    if (MakeDir(Path,PathW,true,DirAttr)!=MKDIR_SUCCESS)
-      Success=false;
+  if (!SkipLastName)
+    if (!IsPathDiv(*PointToLastChar(Path)))
+      if (MakeDir(Path,NULL,true,DirAttr)!=MKDIR_SUCCESS)
+        Success=false;
   return(Success);
 }
 
 
+bool CreatePath(const wchar *Path,bool SkipLastName)
+{
+  if (Path==NULL || *Path==0)
+    return(false);
+
+#if defined(_WIN_ALL) || defined(_EMX)
+  uint DirAttr=0;
+#else
+  uint DirAttr=0777;
+#endif
+  
+  bool Success=true;
+
+  for (const wchar *s=Path;*s!=0;s++)
+  {
+    if (s-Path>=NM)
+      break;
+
+    if (*s==CPATHDIVIDER)
+    {
+      wchar DirName[NM];
+      wcsncpy(DirName,Path,s-Path);
+      DirName[s-Path]=0;
+
+      if (MakeDir(NULL,DirName,true,DirAttr)==MKDIR_SUCCESS)
+      {
+#ifndef GUI
+        char DirNameA[NM];
+        WideToChar(DirName,DirNameA,ASIZE(DirNameA));
+        DirNameA[ASIZE(DirNameA)-1]=0;
+        mprintf(St(MCreatDir),DirNameA);
+        mprintf(" %s",St(MOk));
+#endif
+      }
+      else
+        Success=false;
+    }
+  }
+  if (!SkipLastName)
+    if (!IsPathDiv(*PointToLastChar(Path)))
+      if (MakeDir(NULL,Path,true,DirAttr)!=MKDIR_SUCCESS)
+        Success=false;
+  return(Success);
+}
+
+
+bool CreatePath(const char *Path,const wchar *PathW,bool SkipLastName)
+{
+#ifdef _WIN_ALL
+  // If we are in Windows, let's try Unicode path first. In Unix we do not
+  // need it (Unix MakeDir will fails with Unicode only name).
+  if (PathW!=NULL && *PathW!=0)
+    return(CreatePath(PathW,SkipLastName));
+#endif
+  if (Path!=NULL && *Path!=0)
+    return(CreatePath(Path,SkipLastName));
+  return(false);
+}
+
+
 void SetDirTime(const char *Name,const wchar *NameW,RarTime *ftm,RarTime *ftc,RarTime *fta)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   if (!WinNT())
     return;
 
@@ -126,7 +171,7 @@ void SetDirTime(const char *Name,const wchar *NameW,RarTime *ftm,RarTime *ftc,Ra
     SetFileAttr(Name,NameW,0);
 
   wchar DirNameW[NM];
-  GetWideName(Name,NameW,DirNameW);
+  GetWideName(Name,NameW,DirNameW,ASIZE(DirNameW));
   HANDLE hFile=CreateFileW(DirNameW,GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,
                           NULL,OPEN_EXISTING,FILE_FLAG_BACKUP_SEMANTICS,NULL);
   if (hFile==INVALID_HANDLE_VALUE)
@@ -151,10 +196,10 @@ void SetDirTime(const char *Name,const wchar *NameW,RarTime *ftm,RarTime *ftc,Ra
 
 bool IsRemovable(const char *Name)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   char Root[NM];
   GetPathRoot(Name,Root);
-  int Type=GetDriveType(*Root ? Root:NULL);
+  int Type=GetDriveTypeA(*Root!=0 ? Root:NULL);
   return(Type==DRIVE_REMOVABLE || Type==DRIVE_CDROM);
 #elif defined(_EMX)
   char Drive=etoupper(Name[0]);
@@ -165,21 +210,23 @@ bool IsRemovable(const char *Name)
 }
 
 
+
+
 #ifndef SFX_MODULE
 int64 GetFreeDisk(const char *Name)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   char Root[NM];
   GetPathRoot(Name,Root);
 
   typedef BOOL (WINAPI *GETDISKFREESPACEEX)(
-    LPCTSTR,PULARGE_INTEGER,PULARGE_INTEGER,PULARGE_INTEGER
+    LPCSTR,PULARGE_INTEGER,PULARGE_INTEGER,PULARGE_INTEGER
    );
   static GETDISKFREESPACEEX pGetDiskFreeSpaceEx=NULL;
 
   if (pGetDiskFreeSpaceEx==NULL)
   {
-    HMODULE hKernel=GetModuleHandle("kernel32.dll");
+    HMODULE hKernel=GetModuleHandleW(L"kernel32.dll");
     if (hKernel!=NULL)
       pGetDiskFreeSpaceEx=(GETDISKFREESPACEEX)GetProcAddress(hKernel,"GetDiskFreeSpaceExA");
   }
@@ -195,7 +242,7 @@ int64 GetFreeDisk(const char *Name)
 
   // We are here if we failed to load GetDiskFreeSpaceExA.
   DWORD SectorsPerCluster,BytesPerSector,FreeClusters,TotalClusters;
-  if (!GetDiskFreeSpace(*Root ? Root:NULL,&SectorsPerCluster,&BytesPerSector,&FreeClusters,&TotalClusters))
+  if (!GetDiskFreeSpaceA(*Root ? Root:NULL,&SectorsPerCluster,&BytesPerSector,&FreeClusters,&TotalClusters))
     return(1457664);
   int64 FreeSize=SectorsPerCluster*BytesPerSector;
   FreeSize=FreeSize*FreeClusters;
@@ -215,7 +262,7 @@ int64 GetFreeDisk(const char *Name)
 #elif defined(_UNIX)
   return(1457664);
 #elif defined(_EMX)
-  int Drive=(!isalpha(Name[0]) || Name[1]!=':') ? 0:etoupper(Name[0])-'A'+1;
+  int Drive=IsDiskLetter(Name) ? etoupper(Name[0])-'A'+1:0;
 #ifndef _DJGPP
   if (_osmode == OS2_MODE)
   {
@@ -252,22 +299,31 @@ int64 GetFreeDisk(const char *Name)
 #endif
 
 
+
+
+
 bool FileExist(const char *Name,const wchar *NameW)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
     if (WinNT() && NameW!=NULL && *NameW!=0)
       return(GetFileAttributesW(NameW)!=0xffffffff);
     else
-      return(GetFileAttributes(Name)!=0xffffffff);
+      return(Name!=NULL && GetFileAttributesA(Name)!=0xffffffff);
 #elif defined(ENABLE_ACCESS)
   return(access(Name,0)==0);
 #else
-  struct FindData FD;
+  FindData FD;
   return(FindFile::FastFind(Name,NameW,&FD));
 #endif
 }
 
 
+bool FileExist(const wchar *Name)
+{
+  return FileExist(NULL,Name);
+}
+ 
+
 bool WildFileExist(const char *Name,const wchar *NameW)
 {
   if (IsWildcard(Name,NameW))
@@ -275,7 +331,7 @@ bool WildFileExist(const char *Name,const wchar *NameW)
     FindFile Find;
     Find.SetMask(Name);
     Find.SetMaskW(NameW);
-    struct FindData fd;
+    FindData fd;
     return(Find.Next(&fd));
   }
   return(FileExist(Name,NameW));
@@ -284,7 +340,7 @@ bool WildFileExist(const char *Name,const wchar *NameW)
 
 bool IsDir(uint Attr)
 {
-#if defined (_WIN_32) || defined(_EMX)
+#if defined (_WIN_ALL) || defined(_EMX)
   return(Attr!=0xffffffff && (Attr & 0x10)!=0);
 #endif
 #if defined(_UNIX)
@@ -304,7 +360,7 @@ bool IsUnreadable(uint Attr)
 
 bool IsLabel(uint Attr)
 {
-#if defined (_WIN_32) || defined(_EMX)
+#if defined (_WIN_ALL) || defined(_EMX)
   return((Attr & 8)!=0);
 #else
   return(false);
@@ -328,7 +384,7 @@ bool IsLink(uint Attr)
 
 bool IsDeleteAllowed(uint FileAttr)
 {
-#if defined(_WIN_32) || defined(_EMX)
+#if defined(_WIN_ALL) || defined(_EMX)
   return((FileAttr & (FA_RDONLY|FA_SYSTEM|FA_HIDDEN))==0);
 #else
   return((FileAttr & (S_IRUSR|S_IWUSR))==(S_IRUSR|S_IWUSR));
@@ -338,22 +394,23 @@ bool IsDeleteAllowed(uint FileAttr)
 
 void PrepareToDelete(const char *Name,const wchar *NameW)
 {
-#if defined(_WIN_32) || defined(_EMX)
+#if defined(_WIN_ALL) || defined(_EMX)
   SetFileAttr(Name,NameW,0);
 #endif
 #ifdef _UNIX
-  chmod(Name,S_IRUSR|S_IWUSR|S_IXUSR);
+  if (Name!=NULL)
+    chmod(Name,S_IRUSR|S_IWUSR|S_IXUSR);
 #endif
 }
 
 
 uint GetFileAttr(const char *Name,const wchar *NameW)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
     if (WinNT() && NameW!=NULL && *NameW!=0)
       return(GetFileAttributesW(NameW));
     else
-      return(GetFileAttributes(Name));
+      return(GetFileAttributesA(Name));
 #elif defined(_DJGPP)
   return(_chmod(Name,0));
 #else
@@ -372,11 +429,14 @@ uint GetFileAttr(const char *Name,const wchar *NameW)
 bool SetFileAttr(const char *Name,const wchar *NameW,uint Attr)
 {
   bool Success;
-#ifdef _WIN_32
+#ifdef _WIN_ALL
     if (WinNT() && NameW!=NULL && *NameW!=0)
       Success=SetFileAttributesW(NameW,Attr)!=0;
     else
-      Success=SetFileAttributes(Name,Attr)!=0;
+      if (Name!=NULL)
+        Success=SetFileAttributesA(Name,Attr)!=0;
+      else
+        Success=false;
 #elif defined(_DJGPP)
   Success=_chmod(Name,1,Attr)!=-1;
 #elif defined(_EMX)
@@ -390,75 +450,6 @@ bool SetFileAttr(const char *Name,const wchar *NameW,uint Attr)
 }
 
 
-void ConvertNameToFull(const char *Src,char *Dest)
-{
-#ifdef _WIN_32
-#ifndef _WIN_CE
-  char FullName[NM],*NamePtr;
-  if (GetFullPathName(Src,sizeof(FullName),FullName,&NamePtr))
-    strcpy(Dest,FullName);
-  else
-#endif
-    if (Src!=Dest)
-      strcpy(Dest,Src);
-#else
-  char FullName[NM];
-  if (IsPathDiv(*Src) || IsDiskLetter(Src))
-    strcpy(FullName,Src);
-  else
-  {
-    if (getcwd(FullName,sizeof(FullName))==NULL)
-      *FullName=0;
-    else
-      AddEndSlash(FullName);
-    strcat(FullName,Src);
-  }
-  strcpy(Dest,FullName);
-#endif
-}
-
-
-#ifndef SFX_MODULE
-void ConvertNameToFull(const wchar *Src,wchar *Dest)
-{
-  if (Src==NULL || *Src==0)
-  {
-    *Dest=0;
-    return;
-  }
-#ifdef _WIN_32
-#ifndef _WIN_CE
-  if (WinNT())
-#endif
-  {
-#ifndef _WIN_CE
-    wchar FullName[NM],*NamePtr;
-    if (GetFullPathNameW(Src,sizeof(FullName)/sizeof(FullName[0]),FullName,&NamePtr))
-      strcpyw(Dest,FullName);
-    else
-#endif
-      if (Src!=Dest)
-        strcpyw(Dest,Src);
-  }
-#ifndef _WIN_CE
-  else
-  {
-    char AnsiName[NM];
-    WideToChar(Src,AnsiName);
-    ConvertNameToFull(AnsiName,AnsiName);
-    CharToWide(AnsiName,Dest);
-  }
-#endif
-#else
-  char AnsiName[NM];
-  WideToChar(Src,AnsiName);
-  ConvertNameToFull(AnsiName,AnsiName);
-  CharToWide(AnsiName,Dest);
-#endif
-}
-#endif
-
-
 #ifndef SFX_MODULE
 char *MkTemp(char *Name)
 {
@@ -468,7 +459,7 @@ char *MkTemp(char *Name)
   int Random=clock();
   for (int Attempt=0;;Attempt++)
   {
-    sprintf(Name+Length-6,"%06u",Random+Attempt);
+    sprintf(Name+Length-6,"%06u",(Random+Attempt)%1000000);
     Name[Length-4]='.';
     if (!FileExist(Name))
       break;
@@ -480,6 +471,25 @@ char *MkTemp(char *Name)
 #endif
 
 
+#if defined(_WIN_ALL) && !defined(SFX_MODULE)
+wchar *MkTemp(wchar *Name)
+{
+  size_t Length=wcslen(Name);
+  if (Length<=6)
+    return(NULL);
+  uint Random=clock();
+  for (uint Attempt=0;;Attempt++)
+  {
+    sprintfw(Name+Length-6,7,L"%06u",(Random+Attempt)%1000000);
+    Name[Length-4]='.';
+    if (!FileExist(NULL,Name))
+      break;
+    if (Attempt==1000)
+      return(NULL);
+  }
+  return(Name);
+}
+#endif
 
 
 #ifndef SFX_MODULE
@@ -547,9 +557,11 @@ bool DelFile(const char *Name)
 }
 
 
+
+
 bool DelFile(const char *Name,const wchar *NameW)
 {
-  return(remove(Name)==0);
+  return(Name!=NULL && remove(Name)==0);
 }
 
 
@@ -557,11 +569,11 @@ bool DelFile(const char *Name,const wchar *NameW)
 
 
 
-#if defined(_WIN_32) && !defined(_WIN_CE) && !defined(SFX_MODULE)
+#if defined(_WIN_ALL) && !defined(_WIN_CE) && !defined(SFX_MODULE)
 bool SetFileCompression(char *Name,wchar *NameW,bool State)
 {
   wchar FileNameW[NM];
-  GetWideName(Name,NameW,FileNameW);
+  GetWideName(Name,NameW,FileNameW,ASIZE(FileNameW));
   HANDLE hFile=CreateFileW(FileNameW,FILE_READ_DATA|FILE_WRITE_DATA,
                  FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,
                  FILE_FLAG_BACKUP_SEMANTICS|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
@@ -583,3 +595,4 @@ bool SetFileCompression(char *Name,wchar *NameW,bool State)
 
 
 
+
