@@ -219,7 +219,7 @@
 #define   MExtrPoints        "\n...         %-56s"
 #define   MExtrErrMkDir      "\nCannot create directory %s"
 #define   MExtrPrinting      "\n------ Printing %s\n\n"
-#define   MEncrBadCRC        "\nEncrypted file:  CRC failed in %s (password incorrect ?)"
+#define   MEncrBadCRC        "\nCRC failed in the encrypted file %s. Corrupt file or wrong password."
 #define   MExtrNoFiles       "\nNo files to extract"
 #define   MExtrAllOk         "\nAll OK"
 #define   MExtrTotalErr      "\nTotal errors: %ld"
@@ -298,7 +298,7 @@
 #define   MErrCreateLnk      "\nWARNING: Cannot create link %s"
 #define   MSymLinkExists     "\nWARNING: Symbolic link %s already exists"
 #define   MAskRetryCreate    "\nCannot create %s. Retry ?"
-#define   MListMACHead1      "\n MacOS file type:  %c%c%c%c  ; "
+#define   MListMACHead1      "\n Mac OS file type:  %c%c%c%c  ; "
 #define   MListMACHead2      "file creator:  %c%c%c%c\n"
 #define   MDataBadCRC        "\n%-20s : packed data CRC failed in volume %s"
 #define   MFileRO            "\n%s is read-only"
