@@ -6,8 +6,15 @@
 #define DEFAULT_RECVOLUMES  -10
 
 enum PATH_EXCL_MODE {
-  EXCL_NONE,EXCL_BASEPATH,EXCL_SKIPWHOLEPATH,EXCL_SAVEFULLPATH,
-  EXCL_SKIPABSPATH,EXCL_ABSPATH
+  EXCL_UNCHANGED,      // Process paths as is (default).
+  EXCL_SKIPWHOLEPATH,  // -ep  (exclude the path completely)
+  EXCL_BASEPATH,       // -ep1 (exclude the base part of path)
+  EXCL_SAVEFULLPATH,   // -ep2 (the full path without the disk letter)
+  EXCL_ABSPATH,        // -ep3 (the full path with the disk letter)
+
+  EXCL_SKIPABSPATH     // Works as EXCL_BASEPATH for fully qualified paths
+                       // and as EXCL_UNCHANGED for relative paths.
+                       // Used by WinRAR GUI only.
 };
 
 enum {SOLID_NONE=0,SOLID_NORMAL=1,SOLID_COUNT=2,SOLID_FILEEXT=4,
@@ -66,15 +73,15 @@ class RAROptions
     bool InclAttrSet;
     uint WinSize;
     char TempPath[NM];
-    char SFXModule[NM];
     char ExtrPath[NM];
     wchar ExtrPathW[NM];
     char CommentFile[NM];
+    wchar CommentFileW[NM];
     RAR_CHARSET CommentCharset;
     RAR_CHARSET FilelistCharset;
     char ArcPath[NM];
     wchar ArcPathW[NM];
-    char Password[MAXPASSWORD];
+    wchar Password[MAXPASSWORD];
     bool EncryptHeaders;
     char LogName[NM];
     MESSAGE_TYPE MsgStream;
@@ -109,13 +116,9 @@ class RAROptions
     int Priority;
     int SleepTime;
     bool KeepBroken;
-    bool EraseDisk;
     bool OpenShared;
-    bool ExclEmptyDir;
     bool DeleteFiles;
     bool SyncFiles;
-    bool GenerateArcName;
-    char GenerateMask[80];
     bool ProcessEA;
     bool SaveStreams;
     bool SetCompressedAttr;
@@ -147,6 +150,8 @@ class RAROptions
 
 
 
+
+
 #ifdef RARDLL
     char DllDestName[NM];
     wchar DllDestNameW[NM];
