@@ -3,6 +3,7 @@
 #if !defined(GUI) && !defined(RARDLL)
 int main(int argc, char *argv[])
 {
+
 #ifdef _UNIX
   setlocale(LC_ALL,"");
 #endif
@@ -11,7 +12,7 @@ int main(int argc, char *argv[])
   uni_init(0);
 #endif
 
-#if !defined(_SFX_RTL_) && !defined(_WIN_32)
+#if !defined(_SFX_RTL_) && !defined(_WIN_ALL)
   setbuf(stdout,NULL);
 #endif
 
@@ -24,21 +25,24 @@ int main(int argc, char *argv[])
   RARInitData();
 
 #ifdef SFX_MODULE
-  char ModuleName[NM];
-#ifdef _WIN_32
-  GetModuleFileName(NULL,ModuleName,sizeof(ModuleName));
+  char ModuleNameA[NM];
+  wchar ModuleNameW[NM];
+#ifdef _WIN_ALL
+  GetModuleFileNameW(NULL,ModuleNameW,ASIZE(ModuleNameW));
+  WideToChar(ModuleNameW,ModuleNameA);
 #else
-  strcpy(ModuleName,argv[0]);
+  strcpy(ModuleNameA,argv[0]);
+  *ModuleNameW=0;
 #endif
 #endif
 
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   SetErrorMode(SEM_NOALIGNMENTFAULTEXCEPT|SEM_FAILCRITICALERRORS|SEM_NOOPENFILEERRORBOX);
 
 
 #endif
 
-#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
+#if defined(_WIN_ALL) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
   bool ShutdownOnClose;
 #endif
 
@@ -52,12 +56,12 @@ int main(int argc, char *argv[])
     strcpy(Cmd.Command,"X");
     char *Switch=NULL;
 #ifdef _SFX_RTL_
-    char *CmdLine=GetCommandLine();
+    char *CmdLine=GetCommandLineA();
     if (CmdLine!=NULL && *CmdLine=='\"')
       CmdLine=strchr(CmdLine+1,'\"');
     if (CmdLine!=NULL && (CmdLine=strpbrk(CmdLine," /"))!=NULL)
     {
-      while (isspace(*CmdLine))
+      while (IsSpace(*CmdLine))
         CmdLine++;
       Switch=CmdLine;
     }
@@ -78,7 +82,7 @@ int main(int argc, char *argv[])
           break;
       }
     }
-    Cmd.AddArcName(ModuleName,NULL);
+    Cmd.AddArcName(ModuleNameA,ModuleNameW);
 #else
     if (Cmd.IsConfigEnabled(argc,argv))
     {
@@ -90,7 +94,7 @@ int main(int argc, char *argv[])
 #endif
     Cmd.ParseDone();
 
-#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
+#if defined(_WIN_ALL) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
     ShutdownOnClose=Cmd.Shutdown;
 #endif
 
@@ -120,12 +124,12 @@ int main(int argc, char *argv[])
 #endif
   File::RemoveCreated();
 #if defined(SFX_MODULE) && defined(_DJGPP)
-  _chmod(ModuleName,1,0x20);
+  _chmod(ModuleNameA,1,0x20);
 #endif
 #if defined(_EMX) && !defined(_DJGPP)
   uni_done();
 #endif
-#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
+#if defined(_WIN_ALL) && !defined(SFX_MODULE) && !defined(SHELL_EXT)
   if (ShutdownOnClose)
     Shutdown();
 #endif
