@@ -12,13 +12,19 @@ int stricomp(const char *Str1,const char *Str2);
 int strnicomp(const char *Str1,const char *Str2,size_t N);
 char* RemoveEOL(char *Str);
 char* RemoveLF(char *Str);
-unsigned int loctolower(byte ch);
-unsigned int loctoupper(byte ch);
+wchar* RemoveLF(wchar *Str);
+unsigned char loctolower(unsigned char ch);
+unsigned char loctoupper(unsigned char ch);
 
 char* strncpyz(char *dest, const char *src, size_t maxlen);
-wchar* strncpyzw(wchar *dest, const wchar *src, size_t maxlen);
+wchar* wcsncpyz(wchar *dest, const wchar *src, size_t maxlen);
 
-int etoupper(int ch);
+unsigned char etoupper(unsigned char ch);
+wchar etoupperw(wchar ch);
+
+bool IsDigit(int ch);
+bool IsSpace(int ch);
+bool IsAlpha(int ch);
 
 
 
@@ -28,10 +34,13 @@ bool LowAscii(const wchar *Str);
 
 int stricompc(const char *Str1,const char *Str2);
 #ifndef SFX_MODULE
-int stricompcw(const wchar *Str1,const wchar *Str2);
+int wcsicompc(const wchar *Str1,const wchar *Str2);
 #endif
 
 void itoa(int64 n,char *Str);
 int64 atoil(char *Str);
+void itoa(int64 n,wchar *Str);
+int64 atoil(wchar *Str);
+const wchar* GetWide(const char *Src);
 
 #endif
